@@ -342,6 +342,10 @@ void GroupManagerPrivate::removeTask(TaskPtr task)
         return;
     }
 
+    foreach (LauncherItem *launcher, launchers) {
+        launcher->removeItemIfAssociated(item);
+    }
+
     if (item->parentGroup()) {
         item->parentGroup()->remove(item);
     }
@@ -416,7 +420,11 @@ void GroupManagerPrivate::currentActivityChanged(QString newActivity)
     currentActivity = newActivity;
 
     foreach (LauncherItem *item, launchers) {
-        rootGroups[currentActivity][currentDesktop]->add(item);
+        if (item->shouldShow()) {
+            rootGroups[currentActivity][currentDesktop]->add(item);
+        } else {
+            rootGroups[currentActivity][currentDesktop]->remove(item);
+        }
     }
 
     if (onlyGroupWhenFull) {
@@ -454,7 +462,11 @@ void GroupManagerPrivate::currentDesktopChanged(int newDesktop)
     currentDesktop = newDesktop;
 
     foreach (LauncherItem *item, launchers) {
-        rootGroups[currentActivity][currentDesktop]->add(item);
+        if (item->shouldShow()) {
+            rootGroups[currentActivity][currentDesktop]->add(item);
+        } else {
+            rootGroups[currentActivity][currentDesktop]->remove(item);
+        }
     }
 
     if (onlyGroupWhenFull) {
@@ -527,6 +539,10 @@ void GroupManager::setScreen(int screen)
     d->reloadTasks();
 }
 
+int GroupManager::screen() const
+{
+    return d->currentScreen;
+}
 
 void GroupManagerPrivate::checkScreenChange()
 {
@@ -660,15 +676,10 @@ void GroupManagerPrivate::checkLauncherVisibility(LauncherItem *launcher)
         return;
     }
 
-    typedef QHash<int, TaskGroup *> Metagroup;
-    foreach (Metagroup metagroup, rootGroups) {
-        foreach (TaskGroup *rootGroup, metagroup) {
-            if (launcher->shouldShow()) {
-                rootGroup->add(launcher);
-            } else {
-                rootGroup->remove(launcher);
-            }
-        }
+    if (launcher->shouldShow()) {
+        rootGroups[currentActivity][currentDesktop]->add(launcher);
+    } else {
+        rootGroups[currentActivity][currentDesktop]->remove(launcher);
     }
 }
 
